@@ -13,17 +13,18 @@ from spatialmath.base.animate import *  # lgtm [py/polluting-import]
 from spatialmath.base.graphics import *  # lgtm [py/polluting-import]
 from spatialmath.base.numeric import *  # lgtm [py/polluting-import]
 
-# from spatialmath.base.argcheck import (
-#     assertmatrix,
-#     ismatrix,
-#     getvector,
-#     assertvector,
-#     isvector,
-#     isscalar,
-#     getunit,
-#     isnumberlist,
-#     isvectorlist,
-# )
+from spatialmath.base.argcheck import (
+    assertmatrix,
+    ismatrix,
+    getvector,
+    assertvector,
+    isvector,
+    isscalar,
+    getunit,
+    isnumberlist,
+    isvectorlist,
+)
+
 # from spatialmath.base.quaternions import (
 #     pure,
 #     qnorm,
@@ -124,26 +125,27 @@ from spatialmath.base.numeric import *  # lgtm [py/polluting-import]
 #     homtrans,
 #     rodrigues,
 # )
-# from spatialmath.base.vectors import (
-#     colvec,
-#     unitvec,
-#     unitvec_norm,
-#     norm,
-#     normsq,
-#     isunitvec,
-#     iszerovec,
-#     isunittwist,
-#     isunittwist2,
-#     unittwist,
-#     unittwist_norm,
-#     unittwist2,
-#     angdiff,
-#     removesmall,
-#     cross,
-#     iszero,
-#     wrap_0_2pi,
-#     wrap_mpi_pi,
-# )
+from spatialmath.base.vectors import (
+    colvec,
+    unitvec,
+    unitvec_norm,
+    norm,
+    normsq,
+    isunitvec,
+    iszerovec,
+    isunittwist,
+    isunittwist2,
+    unittwist,
+    unittwist_norm,
+    unittwist2,
+    angdiff,
+    removesmall,
+    cross,
+    iszero,
+    wrap_0_2pi,
+    wrap_mpi_pi,
+)
+
 # from spatialmath.base.symbolic import *
 # from spatialmath.base.animate import Animate, Animate2
 # from spatialmath.base.graphics import (
@@ -183,7 +185,6 @@ __all__ = [
     "getunit",
     "isnumberlist",
     "isvectorlist",
-
     # spatialmath.base.quaternions
     "qpure",
     "qnorm",
@@ -207,7 +208,6 @@ __all__ = [
     "qdotb",
     "qangle",
     "qprint",
-
     # spatialmath.base.transforms2d
     "rot2",
     "trot2",
@@ -224,7 +224,6 @@ __all__ = [
     "xyt2tr",
     "tr2xyt",
     "trinv2",
-
     # spatialmath.base.transforms3d
     "rotx",
     "roty",
@@ -245,6 +244,7 @@ __all__ = [
     "exp2tr",
     "oa2r",
     "oa2tr",
+    "rodrigues",
     "tr2angvec",
     "tr2eul",
     "tr2rpy",
@@ -256,6 +256,7 @@ __all__ = [
     "trinv",
     "tr2delta",
     "tr2jac",
+    "tr2adjoint",
     "rpy2jac",
     "eul2jac",
     "exp2jac",
@@ -269,11 +270,9 @@ __all__ = [
     "x2r",
     "rotvelxform",
     "rotvelxform_inv_dot",
-
     # deprecated
     "angvelxform",
     "angvelxform_dot",
-
     # spatialmath.base.transformsNd
     "t2r",
     "r2t",
@@ -291,8 +290,6 @@ __all__ = [
     "h2e",
     "e2h",
     "homtrans",
-    "rodrigues",
-
     # spatialmath.base.vectors
     "colvec",
     "unitvec",
@@ -316,7 +313,6 @@ __all__ = [
     # spatialmath.base.animate
     "Animate",
     "Animate2",
-
     # spatial.base.graphics
     "plotvol2",
     "plotvol3",
@@ -341,7 +337,6 @@ __all__ = [
     "axes_logic",
     "expand_dims",
     "isnotebook",
-    
     # spatial.base.numeric
     "numjac",
     "numhess",
