@@ -13,12 +13,13 @@ by a 1D NumPy array with 4 elements: s, x, y, z.
 import sys
 import math
 import numpy as np
-from spatialmath import base
+import spatialmath.base as smb
+from spatialmath.base.types import *
 
 _eps = np.finfo(np.float64).eps
 
 
-def qeye():
+def qeye() -> QuaternionArray:
     """
     Create an identity quaternion
 
@@ -38,7 +39,7 @@ def qeye():
     return np.r_[1, 0, 0, 0]
 
 
-def qpure(v):
+def qpure(v: ArrayLike3) -> QuaternionArray:
     """
     Create a pure quaternion
 
@@ -56,11 +57,11 @@ def qpure(v):
         >>> q = qpure([1, 2, 3])
         >>> qprint(q)
     """
-    v = base.getvector(v, 3)
+    v = smb.getvector(v, 3)
     return np.r_[0, v]
 
 
-def qpositive(q):
+def qpositive(q: ArrayLike4) -> QuaternionArray:
     """
     Quaternion with positive scalar part
 
@@ -77,7 +78,7 @@ def qpositive(q):
         return q
 
 
-def qnorm(q):
+def qnorm(q: ArrayLike4) -> float:
     r"""
     Norm of a quaternion
 
@@ -87,7 +88,7 @@ def qnorm(q):
     :rtype: float
 
     Returns the norm (length or magnitude) of the input quaternion which is
-    
+
     .. math::
 
         (s^2 + v_x^2 + v_y^2 + v_z^2)^{1/2}
@@ -101,11 +102,11 @@ def qnorm(q):
     :seealso: :func:`qunit`
 
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     return np.linalg.norm(q)
 
 
-def qunit(q, tol=10):
+def qunit(q: ArrayLike4, tol: Optional[float] = 10) -> UnitQuaternionArray:
     """
     Create a unit quaternion
 
@@ -130,7 +131,7 @@ def qunit(q, tol=10):
 
     :seealso: :func:`qnorm`
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     nm = np.linalg.norm(q)
     if abs(nm) < tol * _eps:
         raise ValueError("cannot normalize (near) zero length quaternion")
@@ -141,10 +142,9 @@ def qunit(q, tol=10):
         return q
     else:
         return -q
-    # return q
 
 
-def qisunit(q, tol=100):
+def qisunit(q: ArrayLike4, tol: Optional[float] = 100) -> bool:
     """
     Test if quaternion has unit length
 
@@ -165,10 +165,30 @@ def qisunit(q, tol=100):
 
     :seealso: :func:`qunit`
     """
-    return base.iszerovec(q, tol=tol)
+    return smb.iszerovec(q, tol=tol)
 
 
-def qisequal(q1, q2, tol=100, unitq=False):
+@overload
+def qisequal(
+    q1: ArrayLike4,
+    q2: ArrayLike4,
+    tol: Optional[float] = 100,
+    unitq: Optional[bool] = False,
+) -> bool:
+    ...
+
+
+@overload
+def qisequal(
+    q1: ArrayLike4,
+    q2: ArrayLike4,
+    tol: Optional[float] = 100,
+    unitq: Optional[bool] = True,
+) -> bool:
+    ...
+
+
+def qisequal(q1, q2, tol: Optional[float] = 100, unitq: Optional[bool] = False):
     """
     Test if quaternions are equal
 
@@ -197,8 +217,8 @@ def qisequal(q1, q2, tol=100, unitq=False):
         >>> qisequal(q1, q2)
         >>> qisequal(q1, q2, unitq=True)
     """
-    q1 = base.getvector(q1, 4)
-    q2 = base.getvector(q2, 4)
+    q1 = smb.getvector(q1, 4)
+    q2 = smb.getvector(q2, 4)
 
     if unitq:
         return (np.sum(np.abs(q1 - q2)) < tol * _eps) or (
@@ -208,7 +228,7 @@ def qisequal(q1, q2, tol=100, unitq=False):
         return np.sum(np.abs(q1 - q2)) < tol * _eps
 
 
-def q2v(q):
+def q2v(q: ArrayLike4) -> R3:
     """
     Convert unit-quaternion to 3-vector
 
@@ -235,14 +255,14 @@ def q2v(q):
     :seealso: :func:`v2q`
 
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     if q[0] >= 0:
         return q[1:4]
     else:
         return -q[1:4]
 
 
-def v2q(v):
+def v2q(v: ArrayLike3) -> UnitQuaternionArray:
     r"""
     Convert 3-vector to unit-quaternion
 
@@ -268,12 +288,12 @@ def v2q(v):
 
     :seealso: :func:`q2v`
     """
-    v = base.getvector(v, 3)
-    s = math.sqrt(1 - np.sum(v ** 2))
+    v = smb.getvector(v, 3)
+    s = math.sqrt(1 - np.sum(v**2))
     return np.r_[s, v]
 
 
-def qqmul(q1, q2):
+def qqmul(q1: ArrayLike4, q2: ArrayLike4) -> QuaternionArray:
     """
     Quaternion multiplication
 
@@ -297,8 +317,8 @@ def qqmul(q1, q2):
     :seealso: qvmul, qinner, vvmul
 
     """
-    q1 = base.getvector(q1, 4)
-    q2 = base.getvector(q2, 4)
+    q1 = smb.getvector(q1, 4)
+    q2 = smb.getvector(q2, 4)
     s1 = q1[0]
     v1 = q1[1:4]
     s2 = q2[0]
@@ -307,7 +327,7 @@ def qqmul(q1, q2):
     return np.r_[s1 * s2 - np.dot(v1, v2), s1 * v2 + s2 * v1 + np.cross(v1, v2)]
 
 
-def qinner(q1, q2):
+def qinner(q1: ArrayLike4, q2: ArrayLike4) -> float:
     """
     Quaternion inner product
 
@@ -316,7 +336,7 @@ def qinner(q1, q2):
     :arg q1: uaternion
     :type q1: array_like(4)
     :return: inner product
-    :rtype: ndarray(4)
+    :rtype: float
 
     This is the inner or dot product of two quaternions, it is the sum of the element-wise
     product.
@@ -338,13 +358,13 @@ def qinner(q1, q2):
     :seealso: qvmul
 
     """
-    q1 = base.getvector(q1, 4)
-    q2 = base.getvector(q2, 4)
+    q1 = smb.getvector(q1, 4)
+    q2 = smb.getvector(q2, 4)
 
     return np.dot(q1, q2)
 
 
-def qvmul(q, v):
+def qvmul(q: ArrayLike4, v: ArrayLike3) -> R3:
     """
     Vector rotation
 
@@ -369,17 +389,16 @@ def qvmul(q, v):
 
     :seealso: qvmul
     """
-    q = base.getvector(q, 4)
-    v = base.getvector(v, 3)
+    q = smb.getvector(q, 4)
+    v = smb.getvector(v, 3)
     qv = qqmul(q, qqmul(qpure(v), qconj(q)))
     return qv[1:4]
 
 
-def vvmul(qa, qb):
+def vvmul(qa: ArrayLike3, qb: ArrayLike3) -> R3:
     """
     Quaternion multiplication
 
-
     :arg qa: left-hand quaternion
     :type qa: : array_like(3)
     :arg qb: right-hand quaternion
@@ -402,10 +421,10 @@ def vvmul(qa, qb):
         >>> vp = vvmul(v1, v2)                 # product using 3-vectors
         >>> qprint(v2q(vp))                    # same answer as Hamilton product
 
-    :seealso: :func:`q2v`, :func:`v2q`, :func:`qvmul`
+    :seealso: :func:`q2v` :func:`v2q` :func:`qvmul`
     """
-    t6 = math.sqrt(1.0 - np.sum(qa ** 2))
-    t11 = math.sqrt(1.0 - np.sum(qb ** 2))
+    t6 = math.sqrt(1.0 - np.sum(qa**2))
+    t11 = math.sqrt(1.0 - np.sum(qb**2))
     return np.r_[
         qa[1] * qb[2] - qb[1] * qa[2] + qb[0] * t6 + qa[0] * t11,
         -qa[0] * qb[2] + qb[0] * qa[2] + qb[1] * t6 + qa[1] * t11,
@@ -413,7 +432,7 @@ def vvmul(qa, qb):
     ]
 
 
-def qpow(q, power):
+def qpow(q: ArrayLike4, power: int) -> QuaternionArray:
     """
     Raise quaternion to a power
 
@@ -443,7 +462,7 @@ def qpow(q, power):
     :seealso: :func:`qqmul`
     :SymPy: supported for ``q`` but not ``power``.
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     if not isinstance(power, int):
         raise ValueError("Power must be an integer")
     qr = qeye()
@@ -456,7 +475,7 @@ def qpow(q, power):
     return qr
 
 
-def qconj(q):
+def qconj(q: ArrayLike4) -> QuaternionArray:
     """
     Quaternion conjugate
 
@@ -475,11 +494,13 @@ def qconj(q):
 
     :SymPy: supported
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     return np.r_[q[0], -q[1:4]]
 
 
-def q2r(q, order="sxyz"):
+def q2r(
+    q: Union[UnitQuaternionArray, ArrayLike4], order: Optional[str] = "sxyz"
+) -> SO3Array:
     """
     Convert unit-quaternion to SO(3) rotation matrix
 
@@ -504,24 +525,29 @@ def q2r(q, order="sxyz"):
     :seealso: :func:`r2q`
 
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     if order == "sxyz":
         s, x, y, z = q
     elif order == "xyzs":
         x, y, z, s = q
     else:
         raise ValueError("order is invalid, must be 'sxyz' or 'xyzs'")
-        
+
     return np.array(
         [
-            [1 - 2 * (y ** 2 + z ** 2), 2 * (x * y - s * z), 2 * (x * z + s * y)],
-            [2 * (x * y + s * z), 1 - 2 * (x ** 2 + z ** 2), 2 * (y * z - s * x)],
-            [2 * (x * z - s * y), 2 * (y * z + s * x), 1 - 2 * (x ** 2 + y ** 2)],
+            [1 - 2 * (y**2 + z**2), 2 * (x * y - s * z), 2 * (x * z + s * y)],
+            [2 * (x * y + s * z), 1 - 2 * (x**2 + z**2), 2 * (y * z - s * x)],
+            [2 * (x * z - s * y), 2 * (y * z + s * x), 1 - 2 * (x**2 + y**2)],
         ]
     )
 
 
-def r2q(R, check=False, tol=100, order="sxyz"):
+def r2q(
+    R: SO3Array,
+    check: Optional[bool] = False,
+    tol: Optional[float] = 100,
+    order: Optional[str] = "sxyz",
+) -> UnitQuaternionArray:
     """
     Convert SO(3) rotation matrix to unit-quaternion
 
@@ -561,7 +587,7 @@ def r2q(R, check=False, tol=100, order="sxyz"):
 
     :seealso: :func:`q2r`
     """
-    if not base.isrot(R, check=check, tol=tol):
+    if not smb.isrot(R, check=check, tol=tol):
         raise ValueError("Argument must be a valid SO(3) matrix")
 
     t12p = (R[0, 1] + R[1, 0]) ** 2
@@ -577,27 +603,80 @@ def r2q(R, check=False, tol=100, order="sxyz"):
     d3 = (-R[0, 0] + R[1, 1] - R[2, 2] + 1) ** 2
     d4 = (-R[0, 0] - R[1, 1] + R[2, 2] + 1) ** 2
 
-    e0 = math.sqrt(d1 + t23m + t13m + t12m) / 4.0
-    e1 = math.sqrt(t23m + d2 + t12p + t13p) / 4.0
-    e2 = math.sqrt(t13m + t12p + d3 + t23p) / 4.0
-    e3 = math.sqrt(t12m + t13p + t23p + d4) / 4.0
+    e = np.array(
+        [
+            math.sqrt(d1 + t23m + t13m + t12m) / 4.0,
+            math.sqrt(t23m + d2 + t12p + t13p) / 4.0,
+            math.sqrt(t13m + t12p + d3 + t23p) / 4.0,
+            math.sqrt(t12m + t13p + t23p + d4) / 4.0,
+        ]
+    )
 
-    # transfer sign from rotation element differences
-    if R[2, 1] < R[1, 2]:
-        e1 = -e1
-    if R[0, 2] < R[2, 0]:
-        e2 = -e2
-    if R[1, 0] < R[0, 1]:
-        e3 = -e3
+    i = np.argmax(e)
+
+    if i == 0:
+        e[1] = math.copysign(e[1], R[2, 1] - R[1, 2])
+        e[2] = math.copysign(e[2], R[0, 2] - R[2, 0])
+        e[3] = math.copysign(e[3], R[1, 0] - R[0, 1])
+    elif i == 1:
+        e[0] = math.copysign(e[0], R[2, 1] - R[1, 2])
+        e[2] = math.copysign(e[2], R[1, 0] + R[0, 1])
+        e[3] = math.copysign(e[3], R[0, 2] + R[2, 0])
+    elif i == 2:
+        e[0] = math.copysign(e[0], R[0, 2] - R[2, 0])
+        e[1] = math.copysign(e[1], R[1, 0] + R[0, 1])
+        e[3] = math.copysign(e[3], R[2, 1] + R[1, 2])
+    else:
+        e[0] = math.copysign(e[0], R[1, 0] - R[0, 1])
+        e[1] = math.copysign(e[1], R[0, 2] + R[2, 0])
+        e[2] = math.copysign(e[1], R[2, 1] + R[1, 2])
 
     if order == "sxyz":
-        return np.r_[e0, e1, e2, e3]
+        return e
     elif order == "xyzs":
-        return np.r_[e1, e2, e3, e0]
+        return e[[1, 2, 3, 0]]
     else:
         raise ValueError("order is invalid, must be 'sxyz' or 'xyzs'")
 
 
+# def r2q_svd(R):
+#     U = np.array(
+#         [
+#             [
+#                 R[0, 0] + R[1, 1] + R[2, 2] + 1,
+#                 R[2, 1] - R[1, 2],
+#                 -R[2, 0] + R[0, 2],
+#                 R[1, 0] - R[0, 1],
+#             ],
+#             [
+#                 R[2, 1] - R[1, 2],
+#                 R[0, 0] - R[1, 1] - R[2, 2] + 1,
+#                 R[1, 0] + R[0, 1],
+#                 R[2, 0] + R[0, 2],
+#             ],
+#             [
+#                 -R[2, 0] + R[0, 2],
+#                 R[1, 0] + R[0, 1],
+#                 -R[0, 0] + R[1, 1] - R[2, 2] + 1,
+#                 R[2, 1] + R[1, 2],
+#             ],
+#             [
+#                 R[1, 0] - R[0, 1],
+#                 R[2, 0] + R[0, 2],
+#                 R[2, 1] + R[1, 2],
+#                 -R[0, 0] - R[1, 1] + R[2, 2] + 1,
+#             ],
+#         ]
+#     )
+
+#     U, S, VT = np.linalg.svd(U)
+
+#     e = U[:, 0]
+#     # if e[0] < -10 * _eps:
+#     #     e = -e
+#     return e
+
+
 # def r2q_old(R, check=False, tol=100):
 #     """
 #     Convert SO(3) rotation matrix to unit-quaternion
@@ -625,9 +704,18 @@ def r2q(R, check=False, tol=100, order="sxyz"):
 
 #     .. note:: Scalar part is always positive.
 
+#     :reference:
+#         - Funda, Taylor, IEEE Trans. Robotics and Automation, 6(3),
+#           June 1990, pp.382-388.  (coding reference)
+#         - Sarabandi, S., and Thomas, F. (March 1, 2019).
+#           "A Survey on the Computation of Quaternions From Rotation Matrices."
+#           ASME. J. Mechanisms Robotics. April 2019; 11(2): 021006. (according to this
+#           paper the algorithm is Hughes' method)
+
+
 #     :seealso: :func:`q2r`
 #     """
-#     if not base.isrot(R, check=check, tol=tol):
+#     if not smb.isrot(R, check=check, tol=tol):
 #         raise ValueError("Argument must be a valid SO(3) matrix")
 
 #     qs = math.sqrt(max(0, np.trace(R) + 1)) / 2.0  # scalar part
@@ -635,22 +723,24 @@ def r2q(R, check=False, tol=100, order="sxyz"):
 #     ky = R[0, 2] - R[2, 0]  # Ax - Nz
 #     kz = R[1, 0] - R[0, 1]  # Ny - Ox
 
+#     # equation (7)
 #     if (R[0, 0] >= R[1, 1]) and (R[0, 0] >= R[2, 2]):
 #         kx1 = R[0, 0] - R[1, 1] - R[2, 2] + 1  # Nx - Oy - Az + 1
 #         ky1 = R[1, 0] + R[0, 1]  # Ny + Ox
 #         kz1 = R[2, 0] + R[0, 2]  # Nz + Ax
-#         add = (kx >= 0)
+#         add = kx >= 0
 #     elif R[1, 1] >= R[2, 2]:
 #         kx1 = R[1, 0] + R[0, 1]  # Ny + Ox
 #         ky1 = R[1, 1] - R[0, 0] - R[2, 2] + 1  # Oy - Nx - Az + 1
 #         kz1 = R[2, 1] + R[1, 2]  # Oz + Ay
-#         add = (ky >= 0)
+#         add = ky >= 0
 #     else:
 #         kx1 = R[2, 0] + R[0, 2]  # Nz + Ax
 #         ky1 = R[2, 1] + R[1, 2]  # Oz + Ay
 #         kz1 = R[2, 2] - R[0, 0] - R[1, 1] + 1  # Az - Nx - Oy + 1
-#         add = (kz >= 0)
+#         add = kz >= 0
 
+#     # equation (8)
 #     if add:
 #         kx = kx + kx1
 #         ky = ky + ky1
@@ -663,12 +753,14 @@ def r2q(R, check=False, tol=100, order="sxyz"):
 #     kv = np.r_[kx, ky, kz]
 #     nm = np.linalg.norm(kv)
 #     if abs(nm) < tol * _eps:
-#         return eye()
+#         return qeye()
 #     else:
-#         return np.r_[qs, (math.sqrt(1.0 - qs ** 2) / nm) * kv]
+#         return np.r_[qs, (math.sqrt(1.0 - qs**2) / nm) * kv]
 
 
-def qslerp(q0, q1, s, shortest=False):
+def qslerp(
+    q0: ArrayLike4, q1: ArrayLike4, s: float, shortest: Optional[bool] = False
+) -> UnitQuaternionArray:
     """
     Quaternion conjugate
 
@@ -708,8 +800,8 @@ def qslerp(q0, q1, s, shortest=False):
     """
     if not 0 <= s <= 1:
         raise ValueError("s must be in the interval [0,1]")
-    q0 = base.getvector(q0, 4)
-    q1 = base.getvector(q1, 4)
+    q0 = smb.getvector(q0, 4)
+    q1 = smb.getvector(q1, 4)
 
     if s == 0:
         return q0
@@ -737,7 +829,7 @@ def qslerp(q0, q1, s, shortest=False):
         return q0
 
 
-def qrand():
+def qrand() -> UnitQuaternionArray:
     """
     Random unit-quaternion
 
@@ -761,7 +853,7 @@ def qrand():
     ]
 
 
-def qmatrix(q):
+def qmatrix(q: ArrayLike4) -> R4x4:
     """
     Convert quaternion to 4x4 matrix equivalent
 
@@ -788,7 +880,7 @@ def qmatrix(q):
     :seealso: qqmul
 
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     s = q[0]
     x = q[1]
     y = q[2]
@@ -796,7 +888,7 @@ def qmatrix(q):
     return np.array([[s, -x, -y, -z], [x, s, -z, y], [y, z, s, -x], [z, -y, x, s]])
 
 
-def qdot(q, w):
+def qdot(q: ArrayLike4, w: ArrayLike3) -> QuaternionArray:
     """
     Rate of change of unit-quaternion
 
@@ -821,13 +913,13 @@ def qdot(q, w):
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
-    q = base.getvector(q, 4)
-    w = base.getvector(w, 3)
-    E = q[0] * (np.eye(3, 3)) - base.skew(q[1:4])
+    q = smb.getvector(q, 4)
+    w = smb.getvector(w, 3)
+    E = q[0] * (np.eye(3, 3)) - smb.skew(q[1:4])
     return 0.5 * np.r_[-np.dot(q[1:4], w), E @ w]
 
 
-def qdotb(q, w):
+def qdotb(q: ArrayLike4, w: ArrayLike3) -> QuaternionArray:
     """
     Rate of change of unit-quaternion
 
@@ -852,13 +944,13 @@ def qdotb(q, w):
     .. warning:: There is no check that the passed values are unit-quaternions.
 
     """
-    q = base.getvector(q, 4)
-    w = base.getvector(w, 3)
-    E = q[0] * (np.eye(3, 3)) + base.skew(q[1:4])
+    q = smb.getvector(q, 4)
+    w = smb.getvector(w, 3)
+    E = q[0] * (np.eye(3, 3)) + smb.skew(q[1:4])
     return 0.5 * np.r_[-np.dot(q[1:4], w), E @ w]
 
 
-def qangle(q1, q2):
+def qangle(q1: ArrayLike4, q2: ArrayLike4) -> float:
     """
     Angle between two unit-quaternions
 
@@ -891,12 +983,17 @@ def qangle(q1, q2):
     """
     # TODO different methods
 
-    q1 = base.getvector(q1, 4)
-    q2 = base.getvector(q2, 4)
-    return 2.0 * math.atan2(base.norm(q1 - q2), base.norm(q1 + q2))
+    q1 = smb.getvector(q1, 4)
+    q2 = smb.getvector(q2, 4)
+    return 2.0 * math.atan2(smb.norm(q1 - q2), smb.norm(q1 + q2))
 
 
-def qprint(q, delim=("<", ">"), fmt="{: .4f}", file=sys.stdout):
+def qprint(
+    q: Union[ArrayLike4, ArrayLike4],
+    delim: Optional[Tuple[str, str]] = ("<", ">"),
+    fmt: Optional[str] = "{: .4f}",
+    file: Optional[TextIO] = sys.stdout,
+) -> str:
     """
     Format a quaternion
 
@@ -930,7 +1027,7 @@ def qprint(q, delim=("<", ">"), fmt="{: .4f}", file=sys.stdout):
         >>> q = qrand()   # a unit quaternion
         >>> qprint(q, delim=('<<', '>>'))
     """
-    q = base.getvector(q, 4)
+    q = smb.getvector(q, 4)
     template = "# {} #, #, # {}".replace("#", fmt)
     s = template.format(q[0], delim[0], q[1], q[2], q[3], delim[1])
     if file:
