@@ -13,7 +13,8 @@ tuple, numpy array, numpy row vector or numpy column vector.
 
 import math
 import numpy as np
-from spatialmath.base import getvector
+from spatialmath.base.argcheck import getvector
+from spatialmath.base.types import *
 
 try:  # pragma: no cover
     # print('Using SymPy')
@@ -27,87 +28,7 @@ except ImportError:  # pragma: no cover
 _eps = np.finfo(np.float64).eps
 
 
-def colvec(v):
-    """
-    Create a column vector
-
-    :param v: any vector
-    :type v: array_like(n)
-    :return: a column vector
-    :rtype: ndarray(n,1)
-
-    Convert input to a column vector.
-
-    .. runblock:: pycon
-
-        >>> from spatialmath.base import *
-        >>> colvec([1, 2, 3])
-    """
-    v = getvector(v)
-    return np.array(v).reshape((len(v), 1))
-
-
-def unitvec(v):
-    """
-    Create a unit vector
-
-    :param v: any vector
-    :type v: array_like(n)
-    :return: a unit-vector parallel to ``v``.
-    :rtype: ndarray(n)
-    :raises ValueError: for zero length vector
-
-    ``unitvec(v)`` is a vector parallel to `v` of unit length.
-
-    .. runblock:: pycon
-
-        >>> from spatialmath.base import *
-        >>> unitvec([3, 4])
-
-    :seealso: :func:`~numpy.linalg.norm`
-
-    """
-
-    v = getvector(v)
-    n = norm(v)
-
-    if n > 100 * _eps:  # if greater than eps
-        return v / n
-    else:
-        return None
-
-
-def unitvec_norm(v):
-    """
-    Create a unit vector
-
-    :param v: any vector
-    :type v: array_like(n)
-    :return: a unit-vector parallel to ``v`` and the norm
-    :rtype: (ndarray(n), float)
-    :raises ValueError: for zero length vector
-
-    ``unitvec(v)`` is a vector parallel to `v` of unit length.
-
-    .. runblock:: pycon
-
-        >>> from spatialmath.base import *
-        >>> unitvec([3, 4])
-
-    :seealso: :func:`~numpy.linalg.norm`
-
-    """
-
-    v = getvector(v)
-    n = np.linalg.norm(v)
-
-    if n > 100 * _eps:  # if greater than eps
-        return (v / n, n)
-    else:
-        return None, None
-
-
-def norm(v):
+def norm(v: ArrayLikePure) -> float:
     """
     Norm of vector
 
@@ -140,7 +61,7 @@ def norm(v):
         return math.sqrt(sum)
 
 
-def normsq(v):
+def normsq(v: ArrayLikePure) -> float:
     """
     Squared norm of vector
 
@@ -171,7 +92,7 @@ def normsq(v):
     return sum
 
 
-def cross(u, v):
+def cross(u: ArrayLike3, v: ArrayLike3) -> R3:
     """
     Cross product of vectors
 
@@ -201,7 +122,87 @@ def cross(u, v):
     ]
 
 
-def isunitvec(v, tol=10):
+def colvec(v: ArrayLike) -> NDArray:
+    """
+    Create a column vector
+
+    :param v: any vector
+    :type v: array_like(n)
+    :return: a column vector
+    :rtype: ndarray(n,1)
+
+    Convert input to a column vector.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> colvec([1, 2, 3])
+    """
+    v = getvector(v)
+    return np.array(v).reshape((len(v), 1))
+
+
+def unitvec(v: ArrayLike) -> NDArray:
+    """
+    Create a unit vector
+
+    :param v: any vector
+    :type v: array_like(n)
+    :return: a unit-vector parallel to ``v``.
+    :rtype: ndarray(n)
+    :raises ValueError: for zero length vector
+
+    ``unitvec(v)`` is a vector parallel to `v` of unit length.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> unitvec([3, 4])
+
+    :seealso: :func:`~numpy.linalg.norm`
+
+    """
+
+    v = getvector(v)
+    n = norm(v)
+
+    if n > 100 * _eps:  # if greater than eps
+        return v / n
+    else:
+        raise ValueError("zero norm vector")
+
+
+def unitvec_norm(v: ArrayLike, tol: float = 100) -> Tuple[NDArray, float]:
+    """
+    Create a unit vector
+
+    :param v: any vector
+    :type v: array_like(n)
+    :return: a unit-vector parallel to ``v`` and the norm
+    :rtype: (ndarray(n), float)
+    :raises ValueError: for zero length vector
+
+    ``unitvec(v)`` is a vector parallel to `v` of unit length.
+
+    .. runblock:: pycon
+
+        >>> from spatialmath.base import *
+        >>> unitvec([3, 4])
+
+    :seealso: :func:`~numpy.linalg.norm`
+
+    """
+
+    v = getvector(v)
+    nm = norm(v)
+
+    if nm > tol * _eps:  # if greater than eps
+        return (v / nm, nm)
+    else:
+        raise ValueError("zero norm vector")
+
+
+def isunitvec(v: ArrayLike, tol: float = 10) -> bool:
     """
     Test if vector has unit length
 
@@ -220,10 +221,10 @@ def isunitvec(v, tol=10):
 
     :seealso: unit, iszerovec, isunittwist
     """
-    return abs(np.linalg.norm(v) - 1) < tol * _eps
+    return bool(abs(np.linalg.norm(v) - 1) < tol * _eps)
 
 
-def iszerovec(v, tol=10):
+def iszerovec(v: ArrayLike, tol: float = 10) -> bool:
     """
     Test if vector has zero length
 
@@ -242,10 +243,10 @@ def iszerovec(v, tol=10):
 
     :seealso: unit, isunitvec, isunittwist
     """
-    return np.linalg.norm(v) < tol * _eps
+    return bool(np.linalg.norm(v) < tol * _eps)
 
 
-def iszero(v, tol=10):
+def iszero(v: float, tol: float = 10) -> bool:
     """
     Test if scalar is zero
 
@@ -264,10 +265,10 @@ def iszero(v, tol=10):
 
     :seealso: unit, iszerovec, isunittwist
     """
-    return abs(v) < tol * _eps
+    return bool(abs(v) < tol * _eps)
 
 
-def isunittwist(v, tol=10):
+def isunittwist(v: ArrayLike6, tol: float = 10) -> bool:
     r"""
     Test if vector represents a unit twist in SE(2) or SE(3)
 
@@ -301,13 +302,13 @@ def isunittwist(v, tol=10):
     if len(v) == 6:
         # test for SE(3) twist
         return isunitvec(v[3:6], tol=tol) or (
-            np.linalg.norm(v[3:6]) < tol * _eps and isunitvec(v[0:3], tol=tol)
+            bool(np.linalg.norm(v[3:6]) < tol * _eps) and isunitvec(v[0:3], tol=tol)
         )
     else:
         raise ValueError
 
 
-def isunittwist2(v, tol=10):
+def isunittwist2(v: ArrayLike3, tol: float = 10) -> bool:
     r"""
     Test if vector represents a unit twist in SE(2) or SE(3)
 
@@ -346,7 +347,7 @@ def isunittwist2(v, tol=10):
         raise ValueError
 
 
-def unittwist(S, tol=10):
+def unittwist(S: ArrayLike6, tol: float = 10) -> Union[R6, None]:
     """
     Convert twist to unit twist
 
@@ -387,7 +388,7 @@ def unittwist(S, tol=10):
     return S / th
 
 
-def unittwist_norm(S, tol=10):
+def unittwist_norm(S: Union[R6, ArrayLike6], tol: float = 10) -> Tuple[R6, float]:
     """
     Convert twist to unit twist and norm
 
@@ -419,7 +420,7 @@ def unittwist_norm(S, tol=10):
     S = getvector(S, 6)
 
     if iszerovec(S, tol=tol):
-        return (None, None)
+        raise ValueError("zero norm")
 
     v = S[0:3]
     w = S[3:6]
@@ -432,7 +433,7 @@ def unittwist_norm(S, tol=10):
     return (S / th, th)
 
 
-def unittwist2(S):
+def unittwist2(S: ArrayLike3) -> R3:
     """
     Convert twist to unit twist
 
@@ -466,7 +467,7 @@ def unittwist2(S):
     return S / th
 
 
-def unittwist2_norm(S):
+def unittwist2_norm(S: ArrayLike3) -> Tuple[R3, float]:
     """
     Convert twist to unit twist
 
@@ -499,7 +500,8 @@ def unittwist2_norm(S):
 
     return (S / th, th)
 
-def wrap_0_pi(theta):
+
+def wrap_0_pi(theta: ArrayLike) -> Union[float, NDArray]:
     r"""
     Wrap angle to range :math:`[0, \pi]`
 
@@ -510,36 +512,99 @@ def wrap_0_pi(theta):
     This is used to fold angles of colatitude.  If zero is the angle of the
     north pole, colatitude increases to :math:`\pi` at the south pole then
     decreases to :math:`0` as we head back to the north pole.
+
+    :seealso: :func:`wrap_mpi2_pi2` :func:`wrap_0_2pi` :func:`wrap_mpi_pi` :func:`angle_wrap`
     """
-    n = (theta / np.pi)
+    theta = np.abs(theta)
+    n = theta / np.pi
     if isinstance(n, np.ndarray):
         n = n.astype(int)
     else:
-        n = int(n)
-    
-    return np.where(n & 1 == 0, theta - n * np.pi, (n+1) * np.pi - theta)
+        n = np.fix(n).astype(int)
+
+    y = np.where(np.bitwise_and(n, 1) == 0, theta - n * np.pi, (n + 1) * np.pi - theta)
+    if isinstance(y, np.ndarray) and y.size == 1:
+        return float(y)
+    else:
+        return y
 
 
-def wrap_0_2pi(theta):
+def wrap_mpi2_pi2(theta: ArrayLike) -> Union[float, NDArray]:
+    r"""
+    Wrap angle to range :math:`[-\pi/2, \pi/2]`
+
+    :param theta: input angle
+    :type theta: scalar or ndarray
+    :return: angle wrapped into range :math:`[-\pi/2, \pi/2]`
+
+    This is used to fold angles of latitude.
+
+    :seealso: :func:`wrap_0_pi` :func:`wrap_0_2pi` :func:`wrap_mpi_pi` :func:`angle_wrap`
+
+    """
+    theta = getvector(theta)
+    n = theta / np.pi * 2
+    if isinstance(n, np.ndarray):
+        n = n.astype(int)
+    else:
+        n = np.fix(n).astype(int)
+
+    y = np.where(np.bitwise_and(n, 1) == 0, theta - n * np.pi, n * np.pi - theta)
+    if isinstance(y, np.ndarray) and len(y) == 1:
+        return float(y)
+    else:
+        return y
+
+
+def wrap_0_2pi(theta: ArrayLike) -> Union[float, NDArray]:
     r"""
     Wrap angle to range :math:`[0, 2\pi)`
 
     :param theta: input angle
     :type theta: scalar or ndarray
     :return: angle wrapped into range :math:`[0, 2\pi)`
+
+    :seealso: :func:`wrap_mpi_pi` :func:`wrap_0_pi` :func:`wrap_mpi2_pi2` :func:`angle_wrap`
     """
-    return theta - 2.0 * math.pi * np.floor(theta / 2.0 / np.pi)
+    theta = getvector(theta)
+    y = theta - 2.0 * math.pi * np.floor(theta / 2.0 / np.pi)
+    if isinstance(y, np.ndarray) and len(y) == 1:
+        return float(y)
+    else:
+        return y
 
 
-def wrap_mpi_pi(angle):
+def wrap_mpi_pi(theta: ArrayLike) -> Union[float, NDArray]:
     r"""
-    Wrap angle to range :math:`[\-pi, \pi)`
+    Wrap angle to range :math:`[-\pi, \pi)`
 
     :param theta: input angle
     :type theta: scalar or ndarray
     :return: angle wrapped into range :math:`[-\pi, \pi)`
+
+    :seealso: :func:`wrap_0_2pi` :func:`wrap_0_pi` :func:`wrap_mpi2_pi2` :func:`angle_wrap`
     """
-    return np.mod(angle + math.pi, 2 * math.pi) - np.pi
+    theta = getvector(theta)
+    y = np.mod(theta + math.pi, 2 * math.pi) - np.pi
+    if isinstance(y, np.ndarray) and len(y) == 1:
+        return float(y)
+    else:
+        return y
+
+
+# @overload
+# def angdiff(a:ArrayLike):
+#     ...
+
+
+@overload
+def angdiff(a: ArrayLike, b: ArrayLike) -> NDArray:
+    ...
+
+
+@overload
+def angdiff(a: ArrayLike) -> NDArray:
+    ...
 
 
 def angdiff(a, b=None):
@@ -575,25 +640,130 @@ def angdiff(a, b=None):
         >>> angdiff(0.9 * pi, -0.9 * pi) / pi
         >>> angdiff(3 * pi)
 
+    :seealso: :func:`vector_diff` :func:`wrap_mpi_pi`
     """
-    if b is None:
-        return np.mod(a + math.pi, 2 * math.pi) - math.pi
+    a = getvector(a)
+    if b is not None:
+        b = getvector(b)
+        a -= b
+
+    return np.mod(a + math.pi, 2 * math.pi) - math.pi
+
+
+def angle_std(theta: ArrayLike) -> float:
+    r"""
+    Standard deviation of angular values
+
+    :param theta: angular values
+    :type theta: array_like
+    :return: circular standard deviation
+    :rtype: float
+
+    .. math::
+
+        \sigma_{\theta} = \sqrt{-2 \log \| \left[ \frac{\sum \sin \theta_i}{N}, \frac{\sum \sin \theta_i}{N} \right] \|} \in [0, \infty)
+
+    :seealso: :func:`angle_mean`
+    """
+    X = np.cos(theta).mean()
+    Y = np.sin(theta).mean()
+    R = np.sqrt(X**2 + Y**2)
+
+    return np.sqrt(-2 * np.log(R))
+
+
+def angle_mean(theta: ArrayLike) -> float:
+    r"""
+    Mean of angular values
+
+    :param theta: angular values
+    :type v: array_like
+    :return: circular mean
+    :rtype: float
+
+    The circular mean is given by
+
+    .. math::
+
+        \bar{\theta} = \tan^{-1} \frac{\sum \sin \theta_i}{\sum \cos \theta_i} \in [-\pi, \pi)]
+
+    :seealso: :func:`angle_std`
+    """
+    X = np.cos(theta).sum()
+    Y = np.sin(theta).sum()
+    return np.arctan2(Y, X)
+
+
+def angle_wrap(theta: ArrayLike, mode: str = "-pi:pi") -> Union[float, NDArray]:
+    """
+    Generalized angle-wrapping
+
+    :param v: angles to wrap
+    :type v: array_like
+    :param mode: wrapping mode, one of: ``"0:2pi"``, ``"0:pi"``, ``"-pi/2:pi/2"`` or ``"-pi:pi"`` [default]
+    :type mode: str, optional
+    :return: wrapped angles
+    :rtype: ndarray
+
+    .. note:: The modes ``"0:pi"`` and ``"-pi/2:pi/2"`` are used to wrap angles of
+        colatitude and latitude respectively.
+
+    :seealso: :func:`wrap_0_2pi` :func:`wrap_mpi_pi` :func:`wrap_0_pi` :func:`wrap_mpi2_pi2`
+    """
+    if mode == "0:2pi":
+        return wrap_0_2pi(theta)
+    elif mode == "-pi:pi":
+        return wrap_mpi_pi(theta)
+    elif mode == "0:pi":
+        return wrap_0_pi(theta)
+    elif mode == "0:pi":
+        return wrap_mpi2_pi2(theta)
     else:
-        return np.mod(a - b + math.pi, 2 * math.pi) - math.pi
+        raise ValueError("bad method specified")
 
-def angle_std(v):
-    pass
 
-def angle_mean(v):
-    pass
+def vector_diff(v1: ArrayLike, v2: ArrayLike, mode: str) -> NDArray:
+    """
+    Generalized vector differnce
 
-def angle_wrap(v, how='-mpi_pi'):
-    pass
+    :param v1: first vector
+    :type v1: array_like(n)
+    :param v2: second vector
+    :type v2: array_like(n)
+    :param mode: subtraction mode
+    :type mode: str of length n
 
-def angle_diff(v):
-    pass
+    ==============  ====================================
+    mode character  purpose
+    ==============  ====================================
+    r               real number, don't wrap
+    c               angle on circle, wrap to [-π, π)
+    C               angle on circle, wrap to [0, 2π)
+    l               latitude angle, wrap to [-π/2, π/2]
+    L               colatitude angle, wrap to [0, π]
+    ==============  ====================================
 
-def removesmall(v, tol=100):
+    :seealso: :func:`angdiff` :func:`wrap_0_2pi` :func:`wrap_mpi_pi` :func:`wrap_0_pi` :func:`wrap_mpi2_pi2`
+    """
+    v = getvector(v1) - getvector(v2)
+    for i, m in enumerate(mode):
+        if m == "r":
+            pass
+        elif m == "c":
+            v[i] = wrap_mpi_pi(v[i])
+        elif m == "C":
+            v[i] = wrap_0_2pi(v[i])
+        elif m == "l":
+            v[i] = wrap_mpi2_pi2(v[i])
+        elif m == "L":
+            v[i] = wrap_0_pi(v[i])
+        else:
+            raise ValueError("bad mode character")
+
+    return v
+
+
+def removesmall(v: ArrayLike, tol: float = 100) -> NDArray:
     """
     Set small values to zero
 
@@ -616,7 +786,7 @@ def removesmall(v, tol=100):
         >>> print(a[3])
 
     """
-    return np.where(abs(v) < tol * _eps, 0, v)
+    return np.where(np.abs(v) < tol * _eps, 0, v)
 
 
 if __name__ == "__main__":  # pragma: no cover
