@@ -26,22 +26,22 @@ import numpy as np
 from spatialmath.base import argcheck
 from spatialmath import base as base
 from spatialmath.baseposematrix import BasePoseMatrix
-import spatialmath.pose3d as p3
 
 # ============================== SO2 =====================================#
 
 
 class SO2(BasePoseMatrix):
     """
-    SO(2) matrix class
+       SO(2) matrix class
 
-    This subclass represents rotations in 2D space.  Internally it is a 2x2 orthogonal matrix belonging
-    to the group SO(2).
+       This subclass represents rotations in 2D space.  Internally it is a 2x2 orthogonal matrix belonging
+       to the group SO(2).
 
- .. inheritance-diagram:: spatialmath.pose2d.SO2
-    :top-classes: collections.UserList
-    :parts: 1
+    .. inheritance-diagram:: spatialmath.pose2d.SO2
+       :top-classes: collections.UserList
+       :parts: 1
     """
+
     # SO2()  identity matrix
     # SO2(angle, unit)
     # SO2( obj )   # deep copy
@@ -49,7 +49,7 @@ class SO2(BasePoseMatrix):
     # SO2( nplist )  # make from list of numpy objects
 
     # constructor needs to take ndarray -> SO2, or list of ndarray -> SO2
-    def __init__(self, arg=None, *, unit='rad', check=True):
+    def __init__(self, arg=None, *, unit="rad", check=True):
         """
         Construct new SO(2) object
 
@@ -73,11 +73,11 @@ class SO2(BasePoseMatrix):
 
         """
         super().__init__()
-        
+
         if isinstance(arg, SE2):
             self.data = [base.t2r(x) for x in arg.data]
 
-        elif  super().arghandler(arg, check=check):
+        elif super().arghandler(arg, check=check):
             return
 
         elif argcheck.isscalar(arg):
@@ -87,7 +87,7 @@ class SO2(BasePoseMatrix):
             self.data = [base.rot2(x, unit=unit) for x in argcheck.getvector(arg)]
 
         else:
-            raise ValueError('bad argument to constructor')
+            raise ValueError("bad argument to constructor")
 
     @staticmethod
     def _identity():
@@ -104,7 +104,7 @@ class SO2(BasePoseMatrix):
         return (2, 2)
 
     @classmethod
-    def Rand(cls, N=1, arange=(0, 2 * math.pi), unit='rad'):
+    def Rand(cls, N=1, arange=(0, 2 * math.pi), unit="rad"):
         r"""
         Construct new SO(2) with random rotation
 
@@ -125,7 +125,9 @@ class SO2(BasePoseMatrix):
         Rotations are uniform over the specified interval.
 
         """
-        rand = np.random.uniform(low=arange[0], high=arange[1], size=N)  # random values in the range
+        rand = np.random.uniform(
+            low=arange[0], high=arange[1], size=N
+        )  # random values in the range
         return cls([base.rot2(x) for x in argcheck.getunit(rand, unit)])
 
     @classmethod
@@ -199,7 +201,7 @@ class SO2(BasePoseMatrix):
         """
         return self.A[:2, :2]
 
-    def theta(self, unit='rad'):
+    def theta(self, unit="rad"):
         """
         SO(2) as a rotation angle
 
@@ -211,7 +213,7 @@ class SO2(BasePoseMatrix):
         ``x.theta`` is the rotation angle such that `x` is `SO2(x.theta)`.
 
         """
-        if unit == 'deg':
+        if unit == "deg":
             conv = 180.0 / math.pi
         else:
             conv = 1.0
@@ -234,28 +236,29 @@ class SO2(BasePoseMatrix):
 
 # ============================== SE2 =====================================#
 
+
 class SE2(SO2):
     """
-    SE(2) matrix class
+       SE(2) matrix class
 
-    This subclass represents rigid-body motion (pose) in 2D space.  Internally
-    it is a 3x3 homogeneous transformation matrix belonging to the group SE(2).
+       This subclass represents rigid-body motion (pose) in 2D space.  Internally
+       it is a 3x3 homogeneous transformation matrix belonging to the group SE(2).
 
- .. inheritance-diagram:: spatialmath.pose2d.SE2
-    :top-classes: collections.UserList
-    :parts: 1
+    .. inheritance-diagram:: spatialmath.pose2d.SE2
+       :top-classes: collections.UserList
+       :parts: 1
     """
 
     # constructor needs to take ndarray -> SO2, or list of ndarray -> SO2
-    def __init__(self, x=None, y=None, theta=None, *, unit='rad', check=True):
+    def __init__(self, x=None, y=None, theta=None, *, unit="rad", check=True):
         """
         Construct new SE(2) object
 
-        :param unit: angular units 'deg' or 'rad' [default] if applicable 
-        :type unit: str, optional 
-        :param check: check for valid SE(2) elements if applicable, default to True 
-        :type check: bool 
-        :return: SE(2) matrix 
+        :param unit: angular units 'deg' or 'rad' [default] if applicable
+        :type unit: str, optional
+        :param check: check for valid SE(2) elements if applicable, default to True
+        :type check: bool
+        :return: SE(2) matrix
         :rtype: SE2 instance
 
         - ``SE2()`` is an SE2 instance representing a null motion -- the
@@ -306,20 +309,19 @@ class SE2(SO2):
                 self.data = [base.trot2(x[2], t=x[:2], unit=unit)]
 
             else:
-                raise ValueError('bad argument to constructor')
+                raise ValueError("bad argument to constructor")
 
         elif x is not None:
-            
             if y is not None and theta is None:
                 # SE2(x, y)
                 self.data = [base.transl2(x, y)]
-        
+
             elif y is not None and theta is not None:
-                    # SE2(x, y, theta)
-                    self.data = [base.trot2(theta, t=[x, y], unit=unit)]
+                # SE2(x, y, theta)
+                self.data = [base.trot2(theta, t=[x, y], unit=unit)]
 
         else:
-            raise ValueError('bad arguments to constructor')
+            raise ValueError("bad arguments to constructor")
 
     @staticmethod
     def _identity():
@@ -336,7 +338,9 @@ class SE2(SO2):
         return (3, 3)
 
     @classmethod
-    def Rand(cls, N=1, xrange=(-1, 1), yrange=(-1, 1), arange=(0, 2 * math.pi), unit='rad'):  # pylint: disable=arguments-differ
+    def Rand(
+        cls, N=1, xrange=(-1, 1), yrange=(-1, 1), arange=(0, 2 * math.pi), unit="rad"
+    ):  # pylint: disable=arguments-differ
         r"""
         Construct a new random SE(2)
 
@@ -366,10 +370,21 @@ class SE2(SO2):
             10
 
         """
-        x = np.random.uniform(low=xrange[0], high=xrange[1], size=N)  # random values in the range
-        y = np.random.uniform(low=yrange[0], high=yrange[1], size=N)  # random values in the range
-        theta = np.random.uniform(low=arange[0], high=arange[1], size=N)  # random values in the range
-        return cls([base.trot2(t, t=[x, y]) for (t, x, y) in zip(x, y, argcheck.getunit(theta, unit))])
+        x = np.random.uniform(
+            low=xrange[0], high=xrange[1], size=N
+        )  # random values in the range
+        y = np.random.uniform(
+            low=yrange[0], high=yrange[1], size=N
+        )  # random values in the range
+        theta = np.random.uniform(
+            low=arange[0], high=arange[1], size=N
+        )  # random values in the range
+        return cls(
+            [
+                base.trot2(t, t=[x, y])
+                for (t, x, y) in zip(x, y, argcheck.getunit(theta, unit))
+            ]
+        )
 
     @classmethod
     def Exp(cls, S, check=True):  # pylint: disable=arguments-differ
@@ -425,7 +440,9 @@ class SE2(SO2):
         :seealso: :func:`~spatialmath.base.transforms3d.transl`
         :SymPy: supported
         """
-        return cls([base.trot2(_th, unit=unit) for _th in base.getvector(theta)], check=False)
+        return cls(
+            [base.trot2(_th, unit=unit) for _th in base.getvector(theta)], check=False
+        )
 
     @classmethod
     def Tx(cls, x):
@@ -452,7 +469,6 @@ class SE2(SO2):
         """
         return cls([base.transl2(_x, 0) for _x in base.getvector(x)], check=False)
 
-
     @classmethod
     def Ty(cls, y):
         """
@@ -563,21 +579,28 @@ class SE2(SO2):
         z-coordinate is settable.
 
         """
+        from spatialmath.pose3d import SE3
+
         def lift3(x):
             y = np.eye(4)
             y[:2, :2] = x.A[:2, :2]
             y[:2, 3] = x.A[:2, 2]
             y[2, 3] = z
             return y
-        return p3.SE3([lift3(x) for x in self])
+
+        return SE3([lift3(x) for x in self])
 
     def Twist2(self):
         from spatialmath.twist import Twist2
 
         return Twist2(self.log(twist=True))
 
-if __name__ == '__main__':  # pragma: no cover
 
+if __name__ == "__main__":  # pragma: no cover
     import pathlib
 
-    exec(open(pathlib.Path(__file__).parent.parent.absolute() / "tests" / "test_pose2d.py").read())  # pylint: disable=exec-used
+    exec(
+        open(
+            pathlib.Path(__file__).parent.parent.absolute() / "tests" / "test_pose2d.py"
+        ).read()
+    )  # pylint: disable=exec-used
